package com.move4mobile.smartpinsdkexample

import android.content.Intent
import android.os.Bundle
import android.text.InputFilter
import android.view.View
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.ViewModelProvider
import com.move4mobile.smartpinsdk.SmartPinManager
import com.move4mobile.smartpinsdk.interfaces.SPDeviceListener
import com.move4mobile.smartpinsdk.interfaces.SPInitListener
import com.move4mobile.smartpinsdk.interfaces.SPPaymentListener
import com.move4mobile.smartpinsdk.interfaces.SPTestListener
import com.move4mobile.smartpinsdk.models.ErrorCode
import com.move4mobile.smartpinsdk.models.SPError
import com.move4mobile.smartpinsdk.models.SPOrder
import com.move4mobile.smartpinsdk.models.SPPaymentType
import com.move4mobile.smartpinsdkexample.databinding.ActivityMainBinding
import com.move4mobile.smartpinsdkexample.databinding.DialogResultBinding
import kotlinx.android.synthetic.main.activity_main.*
import java.math.BigDecimal


class MainActivity : AppCompatActivity(),
    SPDeviceListener,
    SPPaymentListener {

    private var progressDialog: AlertDialog? = null

    private var isRefund = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val binding = DataBindingUtil.setContentView<ActivityMainBinding>(this, R.layout.activity_main)
        val viewModel = ViewModelProvider(this).get(MainActivityViewModel::class.java)
        binding.viewModel = viewModel
        binding.lifecycleOwner = this

        SmartPinManager.debugMode = true

        SmartPinManager.deviceListener = this
        SmartPinManager.paymentListener = this

        amount.filters = arrayOf<InputFilter>(DecimalDigitsInputFilter(5, 2))

        validate_button.setOnClickListener {
            api_progressbar.visibility = View.VISIBLE

            api_message.visibility = View.GONE
            validate_button.visibility = View.GONE

            val token = if (viewModel.token.isNotEmpty()) viewModel.token else ""

            SmartPinManager.init(false, token, object : SPInitListener {
                override fun onReady() {
                    checkIfValidated()
                }

                override fun onNotReady(error: SPError) {
                    checkIfValidated()
                }
            })
        }

        devices_button.setOnClickListener {
            startActivity(Intent(this, DiscoveredTerminalsActivity::class.java))
        }

        test_button.setOnClickListener {
            SmartPinManager.testConnection(object : SPTestListener {
                override fun onTestSuccessful() {
                    showResultDialog(getString(R.string.test_success), getString(R.string.test_success_message))
                }

                override fun onTestFailed(error: SPError) {
                    showResultDialog(getString(R.string.test_failed), getString(R.string.test_failed_message))
                }
            })
        }

        orders_button.setOnClickListener {
            startActivity(Intent(this, OrdersActivity::class.java))
        }

        sites_button.setOnClickListener {
            startActivity(Intent(this, SitesActivity::class.java))
        }

        terminals_button.setOnClickListener {
            startActivity(Intent(this, TerminalsActivity::class.java))
        }

        refund_toggle.setOnCheckedChangeListener { _, isChecked ->
            isRefund = isChecked
        }

        pin_button.setOnClickListener {
            if (amount.text != null && amount.text.isNotEmpty()) {
                toggleLoadingDialog(true)

                val amount = if (isRefund) BigDecimal(amount.text.toString()).negate() else BigDecimal(amount.text.toString())

                SmartPinManager.startPayment(
                    SPPaymentType.PIN,
                    amount
                )
            }
        }

        cash_button.setOnClickListener {
            if (amount.text != null && amount.text.isNotEmpty()) {
                toggleLoadingDialog(true)

                val amount = if (isRefund) BigDecimal(amount.text.toString()).negate() else BigDecimal(amount.text.toString())

                SmartPinManager.startPayment(
                    SPPaymentType.CASH,
                    amount
                )
            }
        }

        checkIfValidated()
        checkConnectedDevice()
    }

    override fun onConnecting() {
        connection_loader.visibility = View.VISIBLE
        connected_device_name.visibility = View.GONE
    }

    override fun onConnected() {
        connection_loader.visibility = View.GONE
        connected_device_name.visibility = View.VISIBLE

        val device = SmartPinManager.getConnectedDevice()
        connected_device_name.text = device?.name

        checkConnectedDevice()
    }

    override fun onDisconnected() {
        connection_loader.visibility = View.GONE
        connected_device_name.visibility = View.VISIBLE

        connected_device_name.text = ""

        checkButtonStates()
    }

    private fun checkConnectedDevice() {
        val device = SmartPinManager.getConnectedDevice()
        connected_device_name.text = device?.name

        checkButtonStates()
    }

    private fun checkIfValidated() {
        if (SmartPinManager.isValidated()) {
            api_progressbar.visibility = View.GONE
            validate_button.visibility = View.VISIBLE

            api_message.visibility = View.VISIBLE
            api_message.setText(R.string.message_token_validated)
            api_message.setTextColor(ContextCompat.getColor(this@MainActivity, R.color.green))
        } else {
            api_progressbar.visibility = View.GONE
            validate_button.visibility = View.VISIBLE

            api_message.visibility = View.VISIBLE
            api_message.setText(R.string.message_token_not_validated)
            api_message.setTextColor(ContextCompat.getColor(this@MainActivity, R.color.danger))
        }

        checkButtonStates()
    }

    private fun checkButtonStates() {
        orders_button.isEnabled = SmartPinManager.isValidated()
        sites_button.isEnabled = SmartPinManager.isValidated()
        terminals_button.isEnabled = SmartPinManager.isValidated()

        pin_button.isEnabled = SmartPinManager.isValidated() && SmartPinManager.isExternalDeviceAvailable()
        cash_button.isEnabled = SmartPinManager.isValidated()
    }

    override fun onPaymentSuccess(order: SPOrder) {
        amount.text.clear()
        toggleLoadingDialog(false)
        showResultDialog(getString(R.string.successful), getString(R.string.successful_message))
    }

    override fun onPaymentUnknown(error: SPError) {

    }

    override fun onPaymentDeclined() {
        amount.text.clear()
        toggleLoadingDialog(false)
        showResultDialog(getString(R.string.failed), getString(R.string.message_failed))
    }

    override fun onPaymentError(error: SPError) {
        toggleLoadingDialog(false)

        when (error.code) {
            ErrorCode.GENERIC -> {
            }
            ErrorCode.BACKEND_ERROR -> showResultDialog(getString(R.string.generic_error), getString(R.string.generic_error_message))
            ErrorCode.AMOUNT_INVALID -> showResultDialog(getString(R.string.amount_invalid), error.message)
            ErrorCode.REFUND_DISABLED -> showResultDialog(getString(R.string.retour_disabled), getString(R.string.retour_disabled_message))
            ErrorCode.BACKEND_ERROR -> showResultDialog(getString(R.string.generic_error), getString(R.string.generic_error_message))
            ErrorCode.GENERIC -> showResultDialog(getString(R.string.generic_error), getString(R.string.generic_error_message))
        }
    }

    private fun toggleLoadingDialog(toggle: Boolean) {
        if (toggle) {
            val builder = AlertDialog.Builder(this)
            builder.setView(R.layout.dialog_loading)
            builder.setCancelable(false)

            progressDialog = builder.create()
            progressDialog?.show()
        } else {
            progressDialog?.hide()
        }
    }

    private fun showResultDialog(title: String, message: String) {
        val binding = DataBindingUtil.inflate<DialogResultBinding>(layoutInflater, R.layout.dialog_result, null, false)
        binding.title.text = title
        binding.message.text = message

        val builder = AlertDialog.Builder(this)
        builder.setView(binding.root)
        builder.setCancelable(false)
        builder.setNeutralButton(R.string.ok) { dialog, _ ->
            dialog.dismiss()
        }

        progressDialog = builder.create()
        progressDialog?.show()
    }
}
